@@ -1,5 +1,5 @@
 /* Nabi - X Input Method server for hangul
- * Copyright (C) 2003-2008 Choe Hwanjin
+ * Copyright (C) 2003-2009 Choe Hwanjin
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 #include "nabi.h"
 #include "server.h"
+#include "preference.h"
 
 #include "keycapturedialog.h"
 #include "debug.h"
@@ -43,7 +44,6 @@ enum {
 };
 
 static NabiConfig* config = NULL;
-static GtkWidget *hangul_keyboard_list_combo = NULL;
 static GtkTreeModel* trigger_key_model = NULL;
 static GtkTreeModel* off_key_model = NULL;
 static GtkTreeModel* candidate_key_model = NULL;
@@ -225,16 +225,29 @@ on_icon_list_selection_changed(GtkTreeSelection *selection, gpointer data)
 	gtk_tree_model_get(model, &iter, THEMES_LIST_NAME, &theme, -1);
 	nabi_app_set_theme(theme);
 	g_free(theme);
+	nabi_log(4, "preference: set theme: %s\n", config->theme->str);
     }
 }
 
+static void
+on_use_tray_icon_button_toggled(GtkToggleButton* button, gpointer data)
+{
+    gboolean flag = gtk_toggle_button_get_active(button);
+
+    nabi_app_use_tray_icon(flag);
+
+    nabi_log(4, "preference: set use tray icon: %d\n", flag);
+}
+
 static GtkWidget*
-create_theme_page(void)
+create_theme_page(GtkWidget* dialog)
 {
     GtkWidget *page;
     GtkWidget *item;
     GtkWidget *scrolledwindow;
     GtkWidget *treeview;
+    GtkWidget *vbox;
+    GtkWidget *button;
     GtkTreeModel *model;
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
@@ -251,7 +264,11 @@ create_theme_page(void)
     gtk_container_set_border_width(GTK_CONTAINER(scrolledwindow), 0);
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
 					GTK_SHADOW_IN);
-    item = create_pref_item(_("Tray icons"), scrolledwindow, TRUE, TRUE);
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
+
+    item = create_pref_item(_("Tray icons"), vbox, TRUE, TRUE);
     gtk_box_pack_start(GTK_BOX(page), item, TRUE, TRUE, 0);
 
     /* loading themes list */
@@ -260,6 +277,7 @@ create_theme_page(void)
     g_object_unref(G_OBJECT(model));
     gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
     gtk_container_add(GTK_CONTAINER(scrolledwindow), treeview);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-theme", treeview);
 
     /* theme icons */
     /* state None */
@@ -300,10 +318,7 @@ create_theme_page(void)
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
 
-    g_signal_connect(G_OBJECT(selection), "changed",
-		     G_CALLBACK(on_icon_list_selection_changed), NULL);
-
-    path = search_text_in_model(model, THEMES_LIST_NAME, config->theme);
+    path = search_text_in_model(model, THEMES_LIST_NAME, config->theme->str);
     if (path) {
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW(treeview), path, NULL, FALSE);
 	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview),
@@ -311,6 +326,19 @@ create_theme_page(void)
 	gtk_tree_path_free(path);
     }
 
+    // 현재 theme를 선택한후 signal을 연결한다.
+    // 그렇지 않으면 UI를 초기화하는 과정에서 아래 콜백이 불리게 된다.
+    g_signal_connect(G_OBJECT(selection), "changed",
+		     G_CALLBACK(on_icon_list_selection_changed), NULL);
+
+    button = gtk_check_button_new_with_label(_("Use tray icon"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+			         config->use_tray_icon);
+    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 6);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-use-tray-icon", button);
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(on_use_tray_icon_button_toggled), NULL);
+
     return page;
 }
 
@@ -325,6 +353,9 @@ on_hangul_keyboard_changed(GtkComboBox *widget, gpointer data)
 
     if (keyboards[i].id != NULL) {
 	nabi_app_set_hangul_keyboard(keyboards[i].id);
+
+	nabi_log(4, "preference: set hagul keyboard: %s\n",
+		    config->hangul_keyboard->str);
     }
 }
 
@@ -334,14 +365,16 @@ on_latin_keyboard_changed(GtkComboBox *widget, gpointer data)
     int i = gtk_combo_box_get_active(widget);
     NabiKeyboardLayout* item = g_list_nth_data(nabi_server->layouts, i);
     if (item != NULL) {
-	g_free(config->latin_keyboard);
-	config->latin_keyboard = g_strdup(item->name);
+	g_string_assign(config->latin_keyboard, item->name);
 	nabi_server_set_keyboard_layout(nabi_server, item->name);
+
+	nabi_log(4, "preference: set latin keyboard: %s\n",
+		    config->latin_keyboard->str);
     }
 }
 
 static GtkWidget*
-create_keyboard_page(void)
+create_keyboard_page(GtkWidget* dialog)
 {
     GtkWidget *page;
     GtkWidget *item;
@@ -362,14 +395,14 @@ create_keyboard_page(void)
     for (i = 0; keyboards[i].name != NULL; i++) {
 	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
 				  _(keyboards[i].name));
-	if (strcmp(config->hangul_keyboard, keyboards[i].id) == 0) {
+	if (strcmp(config->hangul_keyboard->str, keyboards[i].id) == 0) {
 	    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), i);
 	}
     }
     gtk_size_group_add_widget(size_group, combo_box);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-hangul-keyboard", combo_box);
     g_signal_connect(G_OBJECT(combo_box), "changed",
 		     G_CALLBACK(on_hangul_keyboard_changed), NULL);
-    hangul_keyboard_list_combo = combo_box;
 
     item = create_pref_item(_("Hangul keyboard"), combo_box, FALSE, FALSE);
     gtk_box_pack_start(GTK_BOX(page), item, FALSE, TRUE, 0);
@@ -383,7 +416,7 @@ create_keyboard_page(void)
 	NabiKeyboardLayout* layout = list->data;
 	if (layout != NULL) {
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), layout->name);
-	    if (strcmp(config->latin_keyboard, layout->name) == 0) {
+	    if (strcmp(config->latin_keyboard->str, layout->name) == 0) {
 		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), i);
 	    }
 	}
@@ -391,6 +424,7 @@ create_keyboard_page(void)
 	i++;
     }
     gtk_size_group_add_widget(size_group, combo_box);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-latin-keyboard", combo_box);
     g_signal_connect(G_OBJECT(combo_box), "changed",
 		     G_CALLBACK(on_latin_keyboard_changed), NULL);
 
@@ -489,25 +523,32 @@ create_key_list_string_array(GtkTreeModel* model)
 static void
 update_trigger_keys_setting(GtkTreeModel *model)
 {
+    char *joined;
     char **keys = create_key_list_string_array(model);
 
-    g_free(config->trigger_keys);
-    config->trigger_keys = g_strjoinv(",", keys);
+    joined = g_strjoinv(",", keys);
+    g_string_assign(config->trigger_keys, joined);
     nabi_server_set_trigger_keys(nabi_server, keys);
+    g_free(joined);
     g_strfreev(keys);
+
+    nabi_log(4, "preference: set trigger keys: %s\n",
+		config->trigger_keys->str);
 }
 
 static void
 update_off_keys_setting(GtkTreeModel *model)
 {
+    char *joined;
     char **keys = create_key_list_string_array(model);
 
-    g_free(config->off_keys);
-    config->off_keys = g_strjoinv(",", keys);
+    joined = g_strjoinv(",", keys);
+    g_string_assign(config->off_keys, joined);
     nabi_server_set_off_keys(nabi_server, keys);
+    g_free(joined);
     g_strfreev(keys);
 
-    nabi_log(4, "set off keys: %s\n", config->off_keys);
+    nabi_log(4, "preference: set off keys: %s\n", config->off_keys->str);
 }
 
 static void
@@ -624,18 +665,21 @@ on_key_list_remove_button_clicked(GtkWidget *widget,
 static void
 update_candidate_keys_setting(GtkTreeModel *model)
 {
+    char *joined;
     char **keys = create_key_list_string_array(model);
 
-    g_free(config->candidate_keys);
-    config->candidate_keys = g_strjoinv(",", keys);
+    joined = g_strjoinv(",", keys);
+    g_string_assign(config->candidate_keys, joined);
     nabi_server_set_candidate_keys(nabi_server, keys);
+    g_free(joined);
     g_strfreev(keys);
 
-    nabi_log(4, "set candidate keys: %s\n", config->candidate_keys);
+    nabi_log(4, "preference: set candidate keys: %s\n",
+		config->candidate_keys->str);
 }
 
 static GtkWidget*
-create_hangul_page(void)
+create_hangul_page(GtkWidget* dialog)
 {
     GtkWidget *page;
     GtkWidget *item;
@@ -666,10 +710,11 @@ create_hangul_page(void)
     vbox2 = gtk_vbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
 
-    widget = create_key_list_widget(config->trigger_keys);
+    widget = create_key_list_widget(config->trigger_keys->str);
     treeview = g_object_get_data(G_OBJECT(widget), "treeview");
     gtk_box_pack_start(GTK_BOX(vbox2), widget, TRUE, TRUE, 0);
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-trigger-keys", model);
     g_signal_connect_after(G_OBJECT(model), "row-changed",
 			   G_CALLBACK(update_trigger_keys_setting), NULL);
     g_signal_connect_after(G_OBJECT(model), "row-deleted",
@@ -718,10 +763,11 @@ create_hangul_page(void)
     vbox2 = gtk_vbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
 
-    widget = create_key_list_widget(config->off_keys);
+    widget = create_key_list_widget(config->off_keys->str);
     treeview = g_object_get_data(G_OBJECT(widget), "treeview");
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
     off_key_model = model;
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-off-keys", model);
     gtk_box_pack_start(GTK_BOX(vbox2), widget, TRUE, TRUE, 0);
     g_signal_connect_after(G_OBJECT(model), "row-changed",
 			   G_CALLBACK(update_off_keys_setting), NULL);
@@ -776,7 +822,7 @@ on_candidate_font_button_clicked(GtkWidget *button, gpointer data)
 
     dialog = gtk_font_selection_dialog_new(_("Select hanja font"));
     gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dialog),
-					    config->candidate_font);
+					    config->candidate_font->str);
     gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(dialog),
 					       "訓民正音 훈민정음");
     gtk_widget_show_all(dialog);
@@ -786,10 +832,11 @@ on_candidate_font_button_clicked(GtkWidget *button, gpointer data)
 
     if (result == GTK_RESPONSE_OK) {
 	char *font = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dialog));
-	g_free(config->candidate_font);
-	config->candidate_font = g_strdup(font);
+	g_string_assign(config->candidate_font, font);
 	nabi_server_set_candidate_font(nabi_server, font);
 	candidate_font_button_set_labels(button, font);
+	nabi_log(4, "preference: set candidate font: %s\n",
+		    config->candidate_font->str);
     }
     gtk_widget_destroy(dialog);
 }
@@ -801,10 +848,12 @@ on_simplified_chinese_toggled(GtkToggleButton* button, gpointer data)
 
     config->use_simplified_chinese = flag;
     nabi_server_set_simplified_chinese(nabi_server, flag);
+
+    nabi_log(4, "preference: set use simplified chinese: %d\n", flag);
 }
 
 static GtkWidget*
-create_candidate_page(void)
+create_candidate_page(GtkWidget* dialog)
 {
     GtkWidget *page;
     GtkWidget *item;
@@ -832,7 +881,8 @@ create_candidate_page(void)
     g_object_set_data(G_OBJECT(button), "size_label", label);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
     gtk_container_add(GTK_CONTAINER(button), hbox);
-    candidate_font_button_set_labels(button, config->candidate_font);
+    candidate_font_button_set_labels(button, config->candidate_font->str);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-candidate-font", button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(on_candidate_font_button_clicked), NULL);
     item = create_pref_item(_("Hanja Font"), button, FALSE, FALSE);
@@ -846,11 +896,12 @@ create_candidate_page(void)
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-    widget = create_key_list_widget(config->candidate_keys);
+    widget = create_key_list_widget(config->candidate_keys->str);
     treeview = g_object_get_data(G_OBJECT(widget), "treeview");
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
     candidate_key_model = model;
     gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-candidate-keys", model);
     g_signal_connect_after(G_OBJECT(model), "row-changed",
 			   G_CALLBACK(update_candidate_keys_setting), NULL);
     g_signal_connect_after(G_OBJECT(model), "row-deleted",
@@ -879,6 +930,7 @@ create_candidate_page(void)
     button = gtk_check_button_new_with_label(_("Use simplified chinese"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
 			         config->use_simplified_chinese);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-simplified-chinese", button);
     g_signal_connect(G_OBJECT(button), "toggled",
 		     G_CALLBACK(on_simplified_chinese_toggled), NULL);
 
@@ -893,9 +945,10 @@ on_xim_name_changed(GtkEntry* entry, gpointer data)
 {
     const char* name = gtk_entry_get_text(GTK_ENTRY(entry));
 
-    g_free(config->xim_name);
-    config->xim_name = g_strdup(name);
+    g_string_assign(config->xim_name, name);
     nabi_server_set_xim_name(nabi_server, name);
+
+    nabi_log(4, "preference: set xim name: %s\n", name);
 }
 
 static void
@@ -905,6 +958,8 @@ on_event_flow_button_toggled(GtkToggleButton *button, gpointer data)
 
     config->use_dynamic_event_flow = flag;
     nabi_server_set_dynamic_event_flow(nabi_server, flag);
+
+    nabi_log(4, "preference: set use dynamic event flow: %d\n", flag);
 }
 
 static void
@@ -914,6 +969,8 @@ on_commit_by_word_button_toggled(GtkToggleButton *button, gpointer data)
 
     config->commit_by_word = flag;
     nabi_server_set_commit_by_word(nabi_server, flag);
+
+    nabi_log(4, "preference: set commit by word: %d\n", flag);
 }
 
 static void
@@ -923,35 +980,56 @@ on_auto_reorder_button_toggled(GtkToggleButton *button, gpointer data)
 
     config->auto_reorder = flag;
     nabi_server_set_auto_reorder(nabi_server, flag);
+
+    nabi_log(4, "preference: set auto reorder: %d\n", flag);
 }
 
 static void
-on_input_mode_option_changed(GtkComboBox* combo_box, gpointer data)
+on_default_input_mode_button_toggled(GtkToggleButton *button, gpointer data)
 {
-    NabiInputModeOption input_mode_option;
-    char* input_mode_option_str = NULL;
+    gboolean flag = gtk_toggle_button_get_active(button);
 
-    input_mode_option = (NabiInputModeOption)gtk_combo_box_get_active(combo_box);
+    if (flag) {
+	g_string_assign(config->default_input_mode, "compose");
+	nabi_server_set_default_input_mode(nabi_server, NABI_INPUT_MODE_COMPOSE);
+    } else {
+	g_string_assign(config->default_input_mode, "direct");
+	nabi_server_set_default_input_mode(nabi_server, NABI_INPUT_MODE_DIRECT);
+    }
+
+    nabi_log(4, "preference: set default input mode: %s\n",
+		config->default_input_mode->str);
+}
+
+static void
+on_input_mode_scope_changed(GtkComboBox* combo_box, gpointer data)
+{
+    NabiInputModeScope input_mode_scope;
+    char* input_mode_scope_str = NULL;
+
+    input_mode_scope = (NabiInputModeScope)gtk_combo_box_get_active(combo_box);
     
-    switch (input_mode_option) {
+    switch (input_mode_scope) {
     case NABI_INPUT_MODE_PER_DESKTOP:
-	input_mode_option_str = g_strdup("per_desktop");
+	input_mode_scope_str = g_strdup("per_desktop");
 	break;
     case NABI_INPUT_MODE_PER_APPLICATION:
-	input_mode_option_str = g_strdup("per_application");
+	input_mode_scope_str = g_strdup("per_application");
 	break;
     case NABI_INPUT_MODE_PER_IC:
-	input_mode_option_str = g_strdup("per_context");
+	input_mode_scope_str = g_strdup("per_ic");
 	break;
     case NABI_INPUT_MODE_PER_TOPLEVEL:
     default:
-	input_mode_option_str = g_strdup("per_toplevel");
+	input_mode_scope_str = g_strdup("per_toplevel");
 	break;
     }
 
-    g_free(config->input_mode_option);
-    config->input_mode_option = input_mode_option_str;
-    nabi_server_set_input_mode_option(nabi_server, input_mode_option);
+    g_string_assign(config->input_mode_scope, input_mode_scope_str);
+    nabi_server_set_input_mode_scope(nabi_server, input_mode_scope);
+
+    nabi_log(4, "preference: set input mode scope: %s\n",
+		config->input_mode_scope->str);
 }
 
 static void
@@ -960,13 +1038,15 @@ on_preedit_font_changed(GtkFontButton* widget, gpointer data)
     const char* font_desc;
     font_desc = gtk_font_button_get_font_name(widget);
 
-    g_free(config->preedit_font);
-    config->preedit_font = g_strdup(font_desc);
+    g_string_assign(config->preedit_font, font_desc);
     nabi_server_set_preedit_font(nabi_server, font_desc);
+
+    nabi_log(4, "preference: set preedit font: %s\n",
+		config->preedit_font->str);
 }
 
 static GtkWidget*
-create_advanced_page(void)
+create_advanced_page(GtkWidget* dialog)
 {
     GtkWidget *page;
     GtkWidget *item;
@@ -992,8 +1072,9 @@ create_advanced_page(void)
 
     entry = gtk_entry_new();
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 16);
-    gtk_entry_set_text(GTK_ENTRY(entry), config->xim_name);
+    gtk_entry_set_text(GTK_ENTRY(entry), config->xim_name->str);
     gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 6);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-xim-name", entry);
     g_signal_connect(G_OBJECT(entry), "changed",
 		     G_CALLBACK(on_xim_name_changed), NULL);
 
@@ -1004,6 +1085,7 @@ create_advanced_page(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
 			         config->use_dynamic_event_flow);
     gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-use-dynamic-event-flow", button);
     g_signal_connect(G_OBJECT(button), "toggled",
 		     G_CALLBACK(on_event_flow_button_toggled), NULL);
 
@@ -1014,6 +1096,7 @@ create_advanced_page(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
 			         config->commit_by_word);
     gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-commit-by-word", button);
     g_signal_connect(G_OBJECT(button), "toggled",
 		     G_CALLBACK(on_commit_by_word_button_toggled), NULL);
 
@@ -1024,13 +1107,27 @@ create_advanced_page(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
 			         config->auto_reorder);
     gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-auto-reorder", button);
     g_signal_connect(G_OBJECT(button), "toggled",
 		     G_CALLBACK(on_auto_reorder_button_toggled), NULL);
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-    label = gtk_label_new(_("Input mode option: "));
+    button = gtk_check_button_new_with_label(_("Start in hangul mode"));
+    if (strcmp(config->default_input_mode->str, "compose") == 0)
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+    else
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-default-input-mode", button);
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(on_default_input_mode_button_toggled), NULL);
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    label = gtk_label_new(_("Input mode scope: "));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
     combo_box = gtk_combo_box_new_text();
@@ -1042,13 +1139,13 @@ create_advanced_page(void)
 			NABI_INPUT_MODE_PER_TOPLEVEL, _("per toplevel"));
     gtk_combo_box_insert_text(GTK_COMBO_BOX(combo_box),
 			NABI_INPUT_MODE_PER_IC, _("per context"));
-    if (strcmp(config->input_mode_option, "per_desktop") == 0)
+    if (strcmp(config->input_mode_scope->str, "per_desktop") == 0)
 	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
 				 NABI_INPUT_MODE_PER_DESKTOP);
-    else if (strcmp(config->input_mode_option, "per_application") == 0)
+    else if (strcmp(config->input_mode_scope->str, "per_application") == 0)
 	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
 				NABI_INPUT_MODE_PER_APPLICATION);
-    else if (strcmp(config->input_mode_option, "per_context") == 0)
+    else if (strcmp(config->input_mode_scope->str, "per_ic") == 0)
 	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
 				NABI_INPUT_MODE_PER_IC);
     else
@@ -1056,9 +1153,10 @@ create_advanced_page(void)
 				NABI_INPUT_MODE_PER_TOPLEVEL);
 
     gtk_box_pack_start(GTK_BOX(hbox), combo_box, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-input-mode-option", combo_box);
 
     g_signal_connect(G_OBJECT(combo_box), "changed",
-		     G_CALLBACK(on_input_mode_option_changed), NULL);
+		     G_CALLBACK(on_input_mode_scope_changed), NULL);
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -1066,25 +1164,190 @@ create_advanced_page(void)
     label = gtk_label_new(_("Preedit string font: "));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
-    button = gtk_font_button_new_with_font(config->preedit_font);
+    button = gtk_font_button_new_with_font(config->preedit_font->str);
     gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-preedit-font", button);
     g_signal_connect(G_OBJECT(button), "font-set",
 		     G_CALLBACK(on_preedit_font_changed), NULL);
 
     return page;
 }
 
+static void
+on_preference_reset(GtkWidget *button, gpointer data)
+{
+    GObject *dialog = G_OBJECT(data);
+    gpointer p;
+    GtkTreeModel* model;
+    GtkTreeIter iter;
+    GtkTreePath *path;
+    GList* list;
+    const NabiHangulKeyboard* keyboards;
+    int i;
+
+    // theme
+    p = g_object_get_data(dialog, "nabi-pref-theme");
+    if (p != NULL) {
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(p));
+	path = search_text_in_model(model, THEMES_LIST_NAME, "Jini");
+	if (path != NULL) {
+	    gtk_tree_view_set_cursor (GTK_TREE_VIEW(p), path, NULL, FALSE);
+	    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(p),
+					path, NULL, TRUE, 0.5, 0.0);
+	    gtk_tree_path_free(path);
+	}
+    }
+    nabi_app_set_theme("Jini");
+
+    p = g_object_get_data(dialog, "nabi-pref-use-tray-icon");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), TRUE);
+    }
+
+    // keyboard
+    p = g_object_get_data(dialog, "nabi-pref-hangul-keyboard");
+    if (p != NULL) {
+	keyboards = nabi_server_get_hangul_keyboard_list(nabi_server);
+	for (i = 0; keyboards[i].id != NULL; i++) {
+	    if (strcmp(keyboards[i].id, DEFAULT_KEYBOARD) == 0) {
+		gtk_combo_box_set_active(GTK_COMBO_BOX(p), i);
+		break;
+	    }
+	}
+    }
+    nabi_app_set_hangul_keyboard(DEFAULT_KEYBOARD);
+ 
+    p = g_object_get_data(dialog, "nabi-pref-latin-keyboard");
+    if (p != NULL) {
+	i = 0;
+	list = nabi_server->layouts;
+	while (list != NULL) {
+	    NabiKeyboardLayout* layout = list->data;
+	    if (layout != NULL) {
+		if (strcmp(layout->name, "none") == 0) {
+		    gtk_combo_box_set_active(GTK_COMBO_BOX(p), i);
+		    break;
+		}
+	    }
+	    list = g_list_next(list);
+	    i++;
+	}
+    }
+
+    // hangul
+    p = g_object_get_data(dialog, "nabi-pref-trigger-keys");
+    if (p != NULL) {
+	gtk_list_store_clear(GTK_LIST_STORE(p));
+	gtk_list_store_append(GTK_LIST_STORE(p), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(p), &iter, 0, "Hangul", -1);
+	gtk_list_store_append(GTK_LIST_STORE(p), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(p), &iter, 0, "Shift+space", -1);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-off-keys");
+    if (p != NULL) {
+	gtk_list_store_clear(GTK_LIST_STORE(p));
+	gtk_list_store_append(GTK_LIST_STORE(p), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(p), &iter, 0, "Escape", -1);
+    }
+
+    // hanja
+    p = g_object_get_data(dialog, "nabi-pref-candidate-font");
+    if (p != NULL) {
+	candidate_font_button_set_labels(GTK_WIDGET(p), "Sans 14");
+    }
+    g_string_assign(config->candidate_font, "Sans 14");
+
+    p = g_object_get_data(dialog, "nabi-pref-candidate-keys");
+    if (p != NULL) {
+	gtk_list_store_clear(GTK_LIST_STORE(p));
+	gtk_list_store_append(GTK_LIST_STORE(p), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(p), &iter, 0, "Hangul_Hanja", -1);
+	gtk_list_store_append(GTK_LIST_STORE(p), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(p), &iter, 0, "F9", -1);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-simplified-chinese");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), FALSE);
+    }
+
+    // advanced
+    p = g_object_get_data(dialog, "nabi-pref-xim-name");
+    if (p != NULL) {
+	gtk_entry_set_text(GTK_ENTRY(p), PACKAGE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-use-dynamic-event-flow");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), TRUE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-commit-by-word");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), FALSE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-auto-reorder");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), TRUE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-default-input-mode");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), FALSE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-input-mode-option");
+    if (p != NULL) {
+	gtk_combo_box_set_active(GTK_COMBO_BOX(p), NABI_INPUT_MODE_PER_TOPLEVEL);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-preedit-font");
+    if (p != NULL) {
+	gtk_font_button_set_font_name(GTK_FONT_BUTTON(p), "Sans 9");
+	g_signal_emit_by_name(p, "font-set", NULL);
+    }
+}
+
 static void
 on_preference_destroy(GtkWidget *dialog, gpointer data)
 {
     nabi_app_save_config();
 
-    hangul_keyboard_list_combo = NULL;
     trigger_key_model = NULL;
     off_key_model = NULL;
     candidate_key_model = NULL;
 }
 
+static GtkWidget* reset_button_create(void)
+{
+    GtkWidget *button;
+    GtkWidget *box;
+    GtkWidget *align;
+    GtkWidget *label;
+    GtkWidget *image;
+    gint image_spacing = 3;
+
+    button = gtk_button_new();
+    gtk_widget_style_get(GTK_WIDGET(button), "image-spacing", &image_spacing, NULL);
+
+    box = gtk_hbox_new(FALSE, image_spacing);
+
+    image = gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
+    gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
+
+    label = gtk_label_new_with_mnemonic(_("_Reset"));
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
+
+    align = gtk_alignment_new(0.5, 0.5, 0, 0);
+    gtk_container_add(GTK_CONTAINER(button), align);
+    gtk_container_add(GTK_CONTAINER(align), box);
+
+    return button;
+}
+
 GtkWidget*
 preference_window_create(void)
 {
@@ -1092,13 +1355,15 @@ preference_window_create(void)
     GtkWidget *vbox;
     GtkWidget *notebook;
     GtkWidget *label;
+    GtkWidget *button;
     GtkWidget *child;
+    GtkWidget *action_area;
     GList *list;
 
     config = nabi->config;
 
     dialog = gtk_dialog_new_with_buttons(_("Nabi Preferences"),
-					 NULL,
+					 GTK_WINDOW(nabi->palette),
 					 GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
 					 GTK_STOCK_CLOSE,
 					 GTK_RESPONSE_CLOSE,
@@ -1110,28 +1375,28 @@ preference_window_create(void)
 
     /* icons */
     label = gtk_label_new(_("Tray"));
-    child = create_theme_page();
+    child = create_theme_page(dialog);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);
 
     if (nabi_server != NULL) {
 	/* keyboard */
 	label = gtk_label_new(_("Keyboard"));
-	child = create_keyboard_page();
+	child = create_keyboard_page(dialog);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);
 
 	/* key */
 	label = gtk_label_new(_("Hangul"));
-	child = create_hangul_page();
+	child = create_hangul_page(dialog);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);
 
 	/* candidate */
 	label = gtk_label_new(_("Hanja"));
-	child = create_candidate_page();
+	child = create_candidate_page(dialog);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);
 
 	/* advanced */
 	label = gtk_label_new(_("Advanced"));
-	child = create_advanced_page();
+	child = create_advanced_page(dialog);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);
     }
 
@@ -1145,7 +1410,10 @@ preference_window_create(void)
     gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 300);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 
-    list = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
+    action_area = GTK_DIALOG(dialog)->action_area;
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area), GTK_BUTTONBOX_END);
+
+    list = gtk_container_get_children(GTK_CONTAINER(action_area));
     if (list != NULL) {
 	GList *child = g_list_last(list);
 	if (child != NULL && child->data != NULL)
@@ -1153,23 +1421,13 @@ preference_window_create(void)
 	g_list_free(list);
     }
 
+    button = reset_button_create();
+    gtk_box_pack_end(GTK_BOX(action_area), button, FALSE, FALSE, 0);
+    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(action_area), button, TRUE);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(on_preference_reset), dialog);
+
     gtk_widget_show_all(dialog);
 
     return dialog;
 }
-
-void
-preference_window_update(void)
-{
-    /* hangul keyboard list update */
-    if (hangul_keyboard_list_combo != NULL) {
-	int i;
-	const NabiHangulKeyboard* keyboards;
-	keyboards = nabi_server_get_hangul_keyboard_list(nabi_server);
-	for (i = 0; keyboards[i].name != NULL; i++) {
-	    if (strcmp(config->hangul_keyboard, keyboards[i].id) == 0) {
-		gtk_combo_box_set_active(GTK_COMBO_BOX(hangul_keyboard_list_combo), i);
-	    }
-	}
-    }
-}
