@@ -1,5 +1,5 @@
 /* Nabi - X Input Method server for hangul
- * Copyright (C) 2003-2008 Choe Hwanjin
+ * Copyright (C) 2003-2009 Choe Hwanjin
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@ static NabiHangulKeyboard hangul_keyboard_list[] = {
     { "39", N_("3 set 390") },
     { "3s", N_("3 set no-shift") },
     { "3y", N_("3 set yetguel") },
+    { "ro", N_("Romaja") },
     { NULL, NULL }
 };
 
@@ -159,8 +160,9 @@ nabi_server_new(Display* display, int screen, const char *name)
     server->dynamic_event_flow = True;
     server->commit_by_word = False;
     server->auto_reorder = True;
-    server->input_mode = NABI_INPUT_MODE_DIRECT;
-    server->input_mode_option = NABI_INPUT_MODE_PER_TOPLEVEL;
+    server->default_input_mode = NABI_INPUT_MODE_DIRECT;
+    server->input_mode = server->default_input_mode;
+    server->input_mode_scope = NABI_INPUT_MODE_PER_TOPLEVEL;
     server->output_mode = NABI_OUTPUT_SYLLABLE;
 
     /* hanja */
@@ -896,7 +898,7 @@ nabi_server_toggle_input_mode(NabiServer* server)
     if (server == NULL)
 	return;
 
-    if (server->input_mode_option == NABI_INPUT_MODE_PER_DESKTOP) {
+    if (server->input_mode_scope == NABI_INPUT_MODE_PER_DESKTOP) {
 	if (server->input_mode == NABI_INPUT_MODE_DIRECT) {
 	    server->input_mode = NABI_INPUT_MODE_COMPOSE;
 	    nabi_server_set_mode_info(nabi_server, NABI_MODE_INFO_COMPOSE);
@@ -940,10 +942,17 @@ nabi_server_set_auto_reorder(NabiServer* server, Bool flag)
 }
 
 void
-nabi_server_set_input_mode_option(NabiServer* server, NabiInputModeOption flag)
+nabi_server_set_default_input_mode(NabiServer* server, NabiInputMode mode)
 {
     if (server != NULL)
-	server->input_mode_option = flag;
+	server->default_input_mode = mode;
+}
+
+void
+nabi_server_set_input_mode_scope(NabiServer* server, NabiInputModeScope scope)
+{
+    if (server != NULL)
+	server->input_mode_scope = scope;
 }
 
 void
