@@ -1,5 +1,5 @@
 /* Nabi - X Input Method server for hangul
- * Copyright (C) 2003-2008 Choe Hwanjin
+ * Copyright (C) 2003-2009 Choe Hwanjin
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include "server.h"
 #include "conf.h"
 #include "handlebox.h"
+#include "preference.h"
 
 #include "default-icons.h"
 
@@ -71,16 +72,12 @@ typedef struct _NabiTrayIcon {
     NabiStateIcon* state;
 } NabiTrayIcon;
 
-/* from preference.c */
-GtkWidget* preference_window_create(void);
-void preference_window_update(void);
-
 static GtkWidget *about_dialog = NULL;
 static GtkWidget *preference_dialog = NULL;
+static GtkWidget *hide_palette_menuitem = NULL;
 
 static void nabi_app_load_base_icons();
 static void nabi_app_update_tooltips();
-static void nabi_app_update_keyboard_button();
 
 static void nabi_state_icon_load(NabiStateIcon* state, int w, int h);
 
@@ -89,8 +86,9 @@ static void nabi_palette_show(NabiPalette* palette);
 static void nabi_palette_hide(NabiPalette* palette);
 static void nabi_palette_destroy(NabiPalette* palette);
 
-static gboolean nabi_create_tray_icon(gpointer data);
+static gboolean nabi_tray_icon_create(gpointer data);
 static void nabi_tray_load_icons(NabiTrayIcon* tray, gint default_size);
+static void nabi_tray_icon_destroy(NabiTrayIcon* tray);
 
 static void remove_event_filter();
 static GtkWidget* create_tray_icon_menu(void);
@@ -112,7 +110,7 @@ load_colors(void)
     colormap = gdk_colormap_get_system();
 
     /* preedit foreground */
-    gdk_color_parse(nabi->config->preedit_fg, &color);
+    gdk_color_parse(nabi->config->preedit_fg->str, &color);
     ret = gdk_colormap_alloc_color(colormap, &color, FALSE, TRUE);
     if (ret)
 	nabi_server->preedit_fg = color;
@@ -126,7 +124,7 @@ load_colors(void)
     }
 
     /* preedit background */
-    gdk_color_parse(nabi->config->preedit_bg, &color);
+    gdk_color_parse(nabi->config->preedit_bg->str, &color);
     ret = gdk_colormap_alloc_color(colormap, &color, FALSE, TRUE);
     if (ret)
 	nabi_server->preedit_bg = color;
@@ -144,20 +142,22 @@ static void
 set_up_keyboard(void)
 {
     /* keyboard layout */
-    if (g_path_is_absolute(nabi->config->keyboard_layouts_file)) {
+    if (g_path_is_absolute(nabi->config->keyboard_layouts_file->str)) {
 	nabi_server_load_keyboard_layout(nabi_server,
-					 nabi->config->keyboard_layouts_file);
+				     nabi->config->keyboard_layouts_file->str);
     } else {
 	char* filename = g_build_filename(NABI_DATA_DIR,
-				  nabi->config->keyboard_layouts_file, NULL);
+			      nabi->config->keyboard_layouts_file->str, NULL);
 	nabi_server_load_keyboard_layout(nabi_server,
 					 filename);
 	g_free(filename);
     }
-    nabi_server_set_keyboard_layout(nabi_server, nabi->config->latin_keyboard);
+    nabi_server_set_keyboard_layout(nabi_server,
+				    nabi->config->latin_keyboard->str);
 
     /* set keyboard */
-    nabi_server_set_hangul_keyboard(nabi_server, nabi->config->hangul_keyboard);
+    nabi_server_set_hangul_keyboard(nabi_server,
+				    nabi->config->hangul_keyboard->str);
 }
 
 void
@@ -286,9 +286,9 @@ set_up_output_mode(void)
 
     mode = NABI_OUTPUT_SYLLABLE;
     if (nabi->config->output_mode != NULL) {
-	if (g_ascii_strcasecmp(nabi->config->output_mode, "jamo") == 0) {
+	if (g_ascii_strcasecmp(nabi->config->output_mode->str, "jamo") == 0) {
 	    mode = NABI_OUTPUT_JAMO;
-	} else if (g_ascii_strcasecmp(nabi->config->output_mode, "manual") == 0) {
+	} else if (g_ascii_strcasecmp(nabi->config->output_mode->str, "manual") == 0) {
 	    mode = NABI_OUTPUT_MANUAL;
 	}
     }
@@ -330,13 +330,14 @@ nabi_app_setup_server(void)
     set_up_keyboard();
     load_colors();
     set_up_output_mode();
-    keys = g_strsplit(nabi->config->trigger_keys, ",", 0);
+    keys = g_strsplit(nabi->config->trigger_keys->str, ",", 0);
     nabi_server_set_trigger_keys(nabi_server, keys);
     g_strfreev(keys);
-    keys = g_strsplit(nabi->config->candidate_keys, ",", 0);
+    keys = g_strsplit(nabi->config->candidate_keys->str, ",", 0);
     nabi_server_set_candidate_keys(nabi_server, keys);
     g_strfreev(keys);
-    nabi_server_set_candidate_font(nabi_server, nabi->config->candidate_font);
+    nabi_server_set_candidate_font(nabi_server,
+				nabi->config->candidate_font->str);
     nabi_server_set_dynamic_event_flow(nabi_server,
 				       nabi->config->use_dynamic_event_flow);
     nabi_server_set_commit_by_word(nabi_server, nabi->config->commit_by_word);
@@ -344,22 +345,31 @@ nabi_app_setup_server(void)
     nabi_server_set_simplified_chinese(nabi_server,
 				       nabi->config->use_simplified_chinese);
 
-    option = nabi->config->input_mode_option;
+    option = nabi->config->default_input_mode->str;
+    if (strcmp(option, "compose") == 0) {
+	nabi_server_set_default_input_mode(nabi_server,
+					   NABI_INPUT_MODE_COMPOSE);
+    } else {
+	nabi_server_set_default_input_mode(nabi_server,
+					   NABI_INPUT_MODE_DIRECT);
+    }
+
+    option = nabi->config->input_mode_scope->str;
     if (strcmp(option, "per_desktop") == 0) {
-	nabi_server_set_input_mode_option(nabi_server,
+	nabi_server_set_input_mode_scope(nabi_server,
 					  NABI_INPUT_MODE_PER_DESKTOP);
     } else if (strcmp(option, "per_application") == 0) {
-	nabi_server_set_input_mode_option(nabi_server,
+	nabi_server_set_input_mode_scope(nabi_server,
 					  NABI_INPUT_MODE_PER_APPLICATION);
     } else if (strcmp(option, "per_ic") == 0) {
-	nabi_server_set_input_mode_option(nabi_server,
+	nabi_server_set_input_mode_scope(nabi_server,
 					  NABI_INPUT_MODE_PER_IC);
     } else {
-	nabi_server_set_input_mode_option(nabi_server,
+	nabi_server_set_input_mode_scope(nabi_server,
 					  NABI_INPUT_MODE_PER_TOPLEVEL);
     }
 
-    nabi_server_set_preedit_font(nabi_server, nabi->config->preedit_font);
+    nabi_server_set_preedit_font(nabi_server, nabi->config->preedit_font->str);
 }
 
 void
@@ -509,7 +519,8 @@ on_tray_icon_destroyed(GtkWidget *widget, gpointer data)
     nabi_tray = NULL;
     nabi_log(1, "tray icon is destroyed\n");
 
-    nabi_palette->source_id = g_idle_add(nabi_create_tray_icon, NULL);
+    if (nabi->config->use_tray_icon)
+	nabi_palette->source_id = g_idle_add(nabi_tray_icon_create, NULL);
     nabi_palette_show(nabi_palette);
 }
 
@@ -518,8 +529,8 @@ on_palette_destroyed(GtkWidget *widget, gpointer data)
 {
     gtk_main_quit();
 
-    if (nabi_tray != NULL && nabi_tray->widget != NULL) {
-	gtk_widget_destroy(GTK_WIDGET(nabi_tray->widget));
+    if (nabi_tray != NULL) {
+	nabi_tray_icon_destroy(nabi_tray);
     }
 
     remove_event_filter();
@@ -570,6 +581,11 @@ on_tray_icon_button_press(GtkWidget *widget,
     static GtkWidget *menu = NULL;
 
     if (event->type == GDK_BUTTON_PRESS) {
+	if (preference_dialog != NULL) {
+	    gtk_window_present(GTK_WINDOW(preference_dialog));
+	    return TRUE;
+	}
+
 	if (menu != NULL)
 	    gtk_widget_destroy(menu);
 	menu = create_tray_icon_menu();
@@ -871,7 +887,13 @@ on_menu_show_palette(GtkWidget *widget, gpointer data)
 static void
 on_menu_hide_palette(GtkWidget *widget, gpointer data)
 {
-    nabi_app_show_palette(FALSE);
+    // tray icon을 사용하지 않는 상태에서는 palette를 hide하면 모든 ui가
+    // 화면에서 사라져 nabi를 컨트롤할 수 없게 된다.
+    // 그래서 tray icon을 사용하는 경우가 아니면 아래 함수가 작동하지 
+    // 않도록 한다.
+    if (nabi->config->use_tray_icon) {
+	nabi_app_show_palette(FALSE);
+    }
 }
 
 static void
@@ -880,11 +902,7 @@ on_menu_keyboard(GtkWidget *widget, gpointer data)
     const char* id = (const char*)data;
 
     nabi_app_set_hangul_keyboard(id);
-    preference_window_update();
     nabi_app_save_config();
-
-    nabi_app_update_keyboard_button();
-    nabi_app_update_tooltips();
 }
 
 static void
@@ -944,7 +962,7 @@ create_tray_icon_menu(void)
 	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	    g_signal_connect(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_keyboard), (gpointer)id);
-	    if (strcmp(id, nabi->config->hangul_keyboard) == 0)
+	    if (strcmp(id, nabi->config->hangul_keyboard->str) == 0)
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
 					       TRUE);
 	    i++;
@@ -1028,10 +1046,13 @@ static void
 install_event_filter(GtkWidget *widget)
 {
     GdkScreen *screen;
+    GdkEventMask mask;
 
     screen = gdk_drawable_get_screen(GDK_DRAWABLE(widget->window));
     nabi->root_window = gdk_screen_get_root_window(screen);
-    gdk_window_set_events(nabi->root_window, GDK_PROPERTY_CHANGE_MASK);
+
+    mask = gdk_window_get_events(nabi->root_window);
+    gdk_window_set_events(nabi->root_window, mask | GDK_PROPERTY_CHANGE_MASK);
     gdk_window_add_filter(nabi->root_window, root_window_event_filter, NULL);
 }
 
@@ -1094,7 +1115,7 @@ static void
 nabi_app_load_base_icons()
 {
     gboolean error = FALSE;
-    const char* theme = nabi->config->theme;
+    const char* theme = nabi->config->theme->str;
 
     if (none_pixbuf != NULL)
 	g_object_unref(G_OBJECT(none_pixbuf));
@@ -1124,13 +1145,12 @@ nabi_app_load_base_icons()
 	gtk_dialog_run(GTK_DIALOG(message));
 	gtk_widget_destroy(message);
 
-	g_free(nabi->config->theme);
-	nabi->config->theme = g_strdup("Jini");
+	g_string_assign(nabi->config->theme, "Jini");
     }
 }
 
 static gboolean
-nabi_create_tray_icon(gpointer data)
+nabi_tray_icon_create(gpointer data)
 {
     NabiTrayIcon* tray;
     GtkWidget *eventbox;
@@ -1183,6 +1203,14 @@ nabi_create_tray_icon(gpointer data)
     return FALSE;
 }
 
+static void
+nabi_tray_icon_destroy(NabiTrayIcon* tray)
+{
+    // widget만 destroy하면 destroy signal에서 나머지 delete 처리를 한다.
+    if (tray->widget != NULL)
+	gtk_widget_destroy(GTK_WIDGET(tray->widget));
+}
+
 static void
 on_palette_item_pressed(GtkWidget* widget, gpointer data)
 {
@@ -1213,6 +1241,7 @@ nabi_app_create_palette(void)
     gtk_window_move(GTK_WINDOW(handlebox), nabi->config->x, nabi->config->y);
     gtk_window_set_keep_above(GTK_WINDOW(handlebox), TRUE);
     gtk_window_stick(GTK_WINDOW(handlebox));
+    gtk_window_set_accept_focus(GTK_WINDOW(handlebox), FALSE);
     g_signal_connect_after(G_OBJECT(handlebox), "realize",
 	    		   G_CALLBACK(on_palette_realized), NULL);
     g_signal_connect(G_OBJECT(handlebox), "destroy",
@@ -1231,7 +1260,7 @@ nabi_app_create_palette(void)
     gtk_widget_show(nabi_palette->state->widget);
 
     current_keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
-				    nabi->config->hangul_keyboard);
+				    nabi->config->hangul_keyboard->str);
     if (current_keyboard_name != NULL) {
 	const NabiHangulKeyboard* keyboards;
 
@@ -1289,6 +1318,7 @@ nabi_app_create_palette(void)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_hide_palette), menuitem);
+    hide_palette_menuitem = menuitem;
 
     menuitem = gtk_separator_menu_item_new();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1302,7 +1332,8 @@ nabi_app_create_palette(void)
     g_signal_connect(G_OBJECT(button), "pressed",
 		     G_CALLBACK(on_palette_item_pressed), menu);
 
-    g_idle_add(nabi_create_tray_icon, NULL);
+    if (nabi->config->use_tray_icon)
+	g_idle_add(nabi_tray_icon_create, NULL);
 
     return handlebox;
 }
@@ -1359,11 +1390,32 @@ nabi_palette_destroy(NabiPalette* palette)
 void
 nabi_app_set_theme(const gchar *name)
 {
-    g_free(nabi->config->theme);
-    nabi->config->theme = g_strdup(name);
+    g_string_assign(nabi->config->theme, name);
 
     nabi_app_load_base_icons();
-    nabi_tray_load_icons(nabi_tray, nabi->icon_size);
+    if (nabi_tray != NULL)
+	nabi_tray_load_icons(nabi_tray, nabi->icon_size);
+}
+
+void
+nabi_app_use_tray_icon(gboolean use)
+{
+    nabi->config->use_tray_icon = use;
+
+    // tray icon이 없는 상태에서 hide palette 메뉴를 작동시키면 tray icon도
+    // 없고 palette도 없게 되므로 nabi를 컨트롤 할수 없게 된다. 따라서 
+    // tray icon이 있는 상태에서만 hide palette 메뉴를 사용할수 있게 한다.
+    gtk_widget_set_sensitive(hide_palette_menuitem, use);
+
+    if (use) {
+	if (nabi_tray == NULL) {
+	    nabi_tray_icon_create(NULL);
+	}
+    } else {
+	if (nabi_tray != NULL) {
+	    nabi_tray_icon_destroy(nabi_tray);
+	}
+    }
 }
 
 void
@@ -1379,12 +1431,21 @@ nabi_app_show_palette(gboolean state)
 void
 nabi_app_set_hangul_keyboard(const char *id)
 {
-    nabi_server_set_hangul_keyboard(nabi_server, id);
-    g_free(nabi->config->hangul_keyboard);
     if (id == NULL)
-	nabi->config->hangul_keyboard = NULL;
+	g_string_assign(nabi->config->hangul_keyboard, DEFAULT_KEYBOARD);
     else
-	nabi->config->hangul_keyboard = g_strdup(id);
+	g_string_assign(nabi->config->hangul_keyboard, id);
+
+    nabi_server_set_hangul_keyboard(nabi_server, id);
+
+    // palette의 메뉴 버튼 업데이트
+    if (nabi->keyboard_button != NULL) {
+	const char* name;
+	name = nabi_server_get_current_keyboard_name(nabi_server);
+	gtk_button_set_label(GTK_BUTTON(nabi->keyboard_button), _(name));
+    }
+
+    nabi_app_update_tooltips();
 }
 
 void
@@ -1394,16 +1455,6 @@ nabi_app_save_config()
 	nabi_config_save(nabi->config);
 }
 
-static void
-nabi_app_update_keyboard_button()
-{
-    if (nabi->keyboard_button != NULL) {
-	const char* name;
-	name = nabi_server_get_current_keyboard_name(nabi_server);
-	gtk_button_set_label(GTK_BUTTON(nabi->keyboard_button), _(name));
-    }
-}
-
 static void
 nabi_app_update_tooltips()
 {
@@ -1411,7 +1462,7 @@ nabi_app_update_tooltips()
 	const char* keyboard_name;
 	char tip_text[256];
 	keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
-					    nabi->config->hangul_keyboard);
+					    nabi->config->hangul_keyboard->str);
 	snprintf(tip_text, sizeof(tip_text), _("Nabi: %s"), _(keyboard_name));
 
 	gtk_tooltips_set_tip(GTK_TOOLTIPS(nabi->tooltips), nabi->tray_icon,
